﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MVPViewState.Business.ComponentInterfaces;
using MVPViewState.Business.Components;
using MVPViewState.Business.ViewInterfaces;

namespace MVPViewState.Business.Presenters
{
    public interface IPresenter
    {
    } ;
    
    /// <summary>
    /// The base class for all presenters
    /// </summary>
    public abstract class BasePresenter<TView>: IPresenter  where TView : class, IView
    {
        private TView _view;
        protected TView View
        {
            set
            {
                if (value == null)
                    throw new ArgumentException("View cannot be set to null");
                _view = value;
            }
            get
            {
                return _view;
            }

        }

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BasePresenter&lt;TView&gt;"/> class.
        /// </summary>
        /// <param name="view">The view.</param>
        protected BasePresenter(TView view)
        {
            View = view;
            View.Load += OnViewLoad;
// ReSharper disable DoNotCallOverridableMethodsInConstructor
            SubscribeViewToEvents();
// ReSharper restore DoNotCallOverridableMethodsInConstructor
        }

        #endregion

        #region Shared External Dependencies


        private IReferenceDataComponent _ReferenceDataComponent ;
        protected IReferenceDataComponent  ReferenceDataComponent 
        {
            get
            {
                if (_ReferenceDataComponent  == null)
                    _ReferenceDataComponent  = new ReferenceDataComponent();
                return _ReferenceDataComponent ;
            }
        }
	

        #endregion

        #region Abstract Methods

        /// <summary>
        /// Subscribes the view to events.
        /// This method has to be implemented in every presenter class
        /// </summary>
        protected abstract void SubscribeViewToEvents();

        /// <summary>
        /// Called when the view is loaded.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected abstract void OnViewLoad(object sender, EventArgs e);

        /// <summary>
        /// Method that should be called to bind the model to the view
        /// </summary>
        protected abstract void UpdateView();

        #endregion


        /// <summary>
        /// Indicating whether the view was originated due to post back.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is postback; otherwise, <c>false</c>.
        /// </value>
        protected bool IsPostBack
        {
            get { return View.IsPostBack; }
        }
    }
}
