﻿
using System;
using System.Collections.Specialized;
using System.Web.UI;
using MVPViewState.Business.Presenters;
using MVPViewState.Business.ViewInterfaces;

namespace MVPViewstate.Code
{
    public abstract class BasePage : Page, IView
    {
        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init"/> event to initialize the page.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            CreatePresenter();
            CreateEventhandlers();
            base.OnInit(e);
        }

        /// <summary>
        /// Inits the presenter. This method should just instanciate and return the page's presenter. 
        /// The presenter will automatically be kept alive due to subscribing to the view's Load-event
        /// </summary>
        protected abstract IPresenter CreatePresenter();


        /// <summary>
        /// Creates the event handlers for the page generated events
        /// Handlers should be of type : ButtonTest.Click += (s, e) => TestButtonClick(null,null);
        /// where TestButtonClick is an event defined on the view,  and ButtonTest is a Button control on the page
        /// </summary>
        protected abstract void CreateEventhandlers();


        #region Page parameters using viewstate

        internal virtual void SetPageParam(string key, string value)
        {
            base.ViewState[key] = value;
        }
        internal void SetPageParam(string key, bool value)
        {
            SetPageParam(key, value.ToString());
        }
        internal void SetPageParam(string key, int value)
        {
            SetPageParam(key, value.ToString());
        }
        internal virtual string GetPageParam(string key)
        {
            string result = default(string);
            foreach (string k in base.ViewState.Keys)
                if (k == key)
                    result = base.ViewState[k] as string;
            return result;
        }
        internal bool GetPageParamAsBool(string key)
        {
            bool result;
            bool.TryParse(GetPageParam(key), out result);
            return result;
        }
        internal int GetPageParamAsInt(string key)
        {
            int result;
            int.TryParse(GetPageParam(key), out result);
            return result;
        }
        #endregion
    }
}