﻿
using System;
using System.Collections.Specialized;
using System.Web.UI;
using System.Web.UI.WebControls;
using MVPViewState.Business.Presenters;
using MVPViewState.Business.ViewInterfaces;

namespace MVPViewstate.Code
{
    public abstract class BasePageWithoutViewstate : BasePage, IView
    {
        private NameValueCollection _pageParameters = new NameValueCollection();
        private HiddenField _pageStateField = new HiddenField() { ID = "__pageState" };    

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init"/> event to initialize the page.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            CreatePageParameters();
            base.OnInit(e);
        }

        /// <summary>
        /// Restores the page paramters from the previous page out of the postdata.
        /// Also adds a hidden field to to the form controls to contain the pagestate in the next postbak
        /// </summary>
        private void CreatePageParameters()
        {
            _pageParameters = ParamHelper.StringToParams(Request.Form["__pageState"]);
            Page.Form.Controls.Add(_pageStateField);
        }

        protected override void Render(HtmlTextWriter writer)
        {
            //persist the pageparameter right before rendering
            _pageStateField.Value = ParamHelper.ParamsToString(_pageParameters);
            base.Render(writer);
        }


        #region Page parameters without using viewstate

        internal override void SetPageParam(string key, string value)
        {
            _pageParameters[key] = value;
        }

        internal override string GetPageParam(string key)
        {
            return _pageParameters[key];
        }

        #endregion
    }
}